<?php
/**
 * @package Image Rotation Repair
 * @version 1.0
 */
/*
Plugin Name: Image Rotation Repair
Description: The Image Rotation Repair plugin simply fixes image orientation based on EXIF data.  This is primarily a patch for mis-oriented images delivered from iPhones.  Functionally it filters all uploads and if EXIF->Orientation is set to a number greater than 1, then the image is resaved with a new orientation before the image is processed by wordpress.
Author: Shane & Peter, Inc.
Version: 1.0
Author URI: http://shaneandpeter.com
*/

if ( !class_exists( 'ImageRotationRepair' ) ) {
	class ImageRotationRepair {
		
		public function __construct() {
			add_filter( 'wp_handle_upload_prefilter', array( $this, 'fixImageRotation' ), 10, 1 );
		}

		public function fixImageRotation( $file ) {		

			if ( is_callable('exif_read_data') ) {
				$exif = @exif_read_data( $file['tmp_name'] );
				if ( isset($exif) && isset($exif['Orientation']) && $exif['Orientation'] > 1 ) { 
					include_once( ABSPATH . 'wp-admin/includes/image-edit.php' );
					$orientation = ( $exif['Orientation'] - 1 ) * 90; 
					//error_log( "Fixing Image Orientation on {$file['name']}. Orientation is being adjusted by $orientation degrees." );
					$suffix = substr( $file['name'], strrpos( $file['name'], '.' ) + 1 ); // I know there's a better way to get a file type / mime_type.
					switch ( $suffix ) {
						case 'jpg':
						case 'jpeg':
							$image = imagecreatefromjpeg( $file['tmp_name'] );
							break;
						case 'png':
							$image = imagecreatefrompng( $file['tmp_name'] );
							break;
						case 'gif':
							$image = imagecreatefromgif( $file['tmp_name'] );
							break;
						default:
							$image = false;
							break;
					}
					if ($image) { 
						$image = _rotate_image_resource( $image, $orientation );
						switch ( $suffix ) {
							case 'jpg':
							case 'jpeg':
								imagejpeg( $image, $file['tmp_name'], apply_filters( 'jpeg_quality', 90, 'edit_image' ) );
								break;
							case 'png':
								imagepng($image, $file['tmp_name'] );
								break;
							case 'gif':
								imagegif($image, $file['tmp_name'] );
								break;
						} // end switch
					} // end if $image
				} // end if $exif
			} // end is_callable('exif_read_data')
	
			return $file;
		}
	}

	new ImageRotationRepair();
}
?>